#ifndef Slash_Tools_Data_h
#define Slash_Tools_Data_h

#include <Slash/Core/ISession.h>
#include <Slash/Data/IProcessor.h>
#include <Slash/Tools/Managers.h>

#include "smanip.h"

namespace Slash {
namespace Data {

inline void collect(Slash::Core::ISession& aSession,const std::string& aWhat,const std::string& aCuts = "") {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aCuts);
  proc->execute("collect",ss);
}

inline void visualize(Slash::Core::ISession& aSession,const std::string& aWhere) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back("where="+aWhere);
  proc->execute("visualize",ss);
}

inline int number(Slash::Core::ISession& aSession) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return 0;
  std::vector<std::string> ss;
  std::string s = proc->execute("number",ss);
  int number;
  if(!Slash::to<int>(s,number)) return 0;
  return number;  
}

inline std::string values(Slash::Core::ISession& aSession,const std::string& aWhat) {
  // returned string is a \n separated list.
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return std::string();
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  return proc->execute("values",ss);
}

inline std::vector<std::string> values_vector(Slash::Core::ISession& aSession,const std::string& aWhat) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return std::vector<std::string>();
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  std::vector<std::string> ws;
  Slash::words(proc->execute("values",ss),"\n",false,ws);
  return ws;
}

inline void dump(Slash::Core::ISession& aSession,const std::string& aFormat = "table") {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back(aFormat);
  proc->execute("dump",ss);
}

inline void filter(Slash::Core::ISession& aSession,const std::string& aWhat,const std::string& aFilter = "") {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aFilter);
  proc->execute("filter",ss);
}

inline void set(Slash::Core::ISession& aSession,const std::string& aWhat,const std::string& aValue) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aValue);
  proc->execute("set",ss);
}

inline void destroy(Slash::Core::ISession& aSession) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  proc->execute("destroy",ss);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void fill_histogram_1D(
 Slash::Core::ISession& aSession
,const std::string& aHistogram
,const std::string& aExpressionX
) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back("tree=MemoryTree");
  ss.push_back("path="+aHistogram);
  ss.push_back("x="+aExpressionX);
  ss.push_back("create=no");
  proc->execute("histogram",ss);
}

inline void fill_histogram_2D(
 Slash::Core::ISession& aSession
,const std::string& aHistogram
,const std::string& aExpressionX
,const std::string& aExpressionY
) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back("tree=MemoryTree");
  ss.push_back("path="+aHistogram);
  ss.push_back("dimension=2");
  ss.push_back("x="+aExpressionX);
  ss.push_back("y="+aExpressionY);
  ss.push_back("create=no");
  proc->execute("histogram",ss);
}

inline void create_tuple(
 Slash::Core::ISession& aSession
,const std::string& aPath
,const std::string& aTitle
) {
  Slash::Data::IProcessor* proc = Slash::accessorManager(aSession);
  if(!proc) return;
  std::vector<std::string> ss;
  ss.push_back("tree=MemoryTree");
  ss.push_back("path="+aPath);
  ss.push_back("title="+aTitle);
  proc->execute("tuple",ss);
}

}}

#endif
