#ifndef amanip_system_h
#define amanip_system_h

#include <string>
#include <cstdlib>

#include "smanip.h"
#include "cstring.h"

namespace amanip {

inline bool isenv(const std::string& a_string){
  const char* env = ::getenv(a_string.c_str());
  return (env?true:false);
}

inline bool getenv(const std::string& a_string,std::string& a_value){
  const char* env = ::getenv(a_string.c_str());
  if(env) {
    a_value = std::string(env?env:"");
    return true;
  } else {
    a_value = "";
    return false;
  }
}

inline bool putenv(const std::string& a_env,const std::string& a_value){
  std::string value = a_env+"="+a_value;
  if(::putenv(strdup(value.c_str()))) return false;
  //check:
  std::string s;
  if(!getenv(a_env,s)) return false;
  if(s!=a_value) return false;
  return true;
}

inline std::string path_separator(){
#ifdef WIN32
  return ";";
#else
  return ":";
#endif
}

inline bool env_append(const std::string& a_list,const std::string& a_value,const std::string& a_sep){
  std::string old_value;
  if(isenv(a_list)) {
    if(!getenv(a_list,old_value)) return false;
  }

  if(old_value.size()) {
    std::vector<std::string> ws;
    words(old_value,a_sep,false,ws);
    // Remove existing one, so that a_value be put at end.
    remove(ws,a_value);
    old_value = to_string(ws,a_sep);
  }

  std::string new_value = old_value;
  if(new_value.size()) new_value += a_sep;
  new_value += a_value;
  if(!putenv(a_list,new_value)) return false;

  return true;
}

inline bool env_path_append(const std::string& a_paths,const std::string& a_path){
  return env_append(a_paths,a_path,path_separator());
}

inline int execute(const std::string& a_string) {
  return ::system(a_string.c_str());
}

}

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h> //usleep
#endif

namespace amanip {

inline void sleep(unsigned int a_milli_sec){
#ifdef WIN32
    ::Sleep(a_milli_sec);
#else
    ::usleep(a_milli_sec*1000); //it is in micro seconds.
#endif
}

}

#endif
